==> agentle-0.0.2.tar.gz: agentle-0.0.2/agentle/generations/tools/tool.py <==
from __future__ import annotations

import inspect
from collections.abc import Callable
from typing import Any, Literal

from rsb.models.base_model import BaseModel
from rsb.models.config_dict import ConfigDict
from rsb.models.field import Field


class Tool[T_Output = Any](BaseModel):
    type: Literal["tool"] = Field(default="tool")
    name: str
    description: str | None = Field(default=None)
    parameters: dict[str, object]
    callable_ref: Callable[..., T_Output] | None = Field(default=None, exclude=True)
    needs_human_confirmation: bool = Field(default=False)

    config = ConfigDict(arbitrary_types_allowed=True, frozen=True)

    @property
    def text(self) -> str:
        return f"Tool: {self.name}\nDescription: {self.description}\nParameters: {self.parameters}"

    def call(self, **kwargs: object) -> T_Output:
        if self.callable_ref is None:
            raise ValueError(
                'Tool is not callable because the "callable_ref" instance variable is not set'
            )

        return self.callable_ref(**kwargs)

    @classmethod
    def from_callable(
        cls,
        _callable: Callable[..., T_Output],
        /,
    ) -> Tool[T_Output]:
        name = getattr(_callable, "__name__", "anonymous_function")
        description = _callable.__doc__ or "No description available"

        # Extrair informações dos parâmetros da função
        parameters: dict[str, object] = {}
        signature = inspect.signature(_callable)

        for param_name, param in signature.parameters.items():
            # Ignorar parâmetros do tipo self/cls para métodos
            if (
                param_name in ("self", "cls")
                and param.kind == inspect.Parameter.POSITIONAL_OR_KEYWORD
            ):
                continue

            param_info: dict[str, object] = {"type": "object"}

            # Adicionar informações de tipo se disponíveis
            if param.annotation != inspect.Parameter.empty:
                param_type = (
                    str(param.annotation).replace("<class '", "").replace("'>", "")
                )
                param_info["type"] = param_type

            # Adicionar valor padrão se disponível
            if param.default != inspect.Parameter.empty:
                param_info["default"] = param.default

            # Determinar se o parâmetro é obrigatório
            if param.default == inspect.Parameter.empty and param.kind in (
                inspect.Parameter.POSITIONAL_ONLY,
                inspect.Parameter.POSITIONAL_OR_KEYWORD,
            ):
                param_info["required"] = True

            parameters[param_name] = param_info

        return cls(
            name=name,
            description=description,
            callable_ref=_callable,
            parameters=parameters,
        )

==> agentle-0.0.4.tar.gz: agentle-0.0.4/agentle/generations/tools/tool.py <==
from __future__ import annotations

import inspect
from collections.abc import Callable
from typing import Any, Literal

from rsb.models.base_model import BaseModel
from rsb.models.config_dict import ConfigDict
from rsb.models.field import Field


class Tool[T_Output = Any](BaseModel):
    type: Literal["tool"] = Field(default="tool")
    name: str
    description: str | None = Field(default=None)
    parameters: dict[str, object]
    callable_ref: Callable[..., T_Output] | None = Field(default=None, exclude=True)
    needs_human_confirmation: bool = Field(default=False)

    model_config = ConfigDict(arbitrary_types_allowed=True, frozen=True)

    @property
    def text(self) -> str:
        return f"Tool: {self.name}\nDescription: {self.description}\nParameters: {self.parameters}"

    def call(self, **kwargs: object) -> T_Output:
        if self.callable_ref is None:
            raise ValueError(
                'Tool is not callable because the "callable_ref" instance variable is not set'
            )

        return self.callable_ref(**kwargs)

    @classmethod
    def from_callable(
        cls,
        _callable: Callable[..., T_Output],
        /,
    ) -> Tool[T_Output]:
        name = getattr(_callable, "__name__", "anonymous_function")
        description = _callable.__doc__ or "No description available"

        # Extrair informações dos parâmetros da função
        parameters: dict[str, object] = {}
        signature = inspect.signature(_callable)

        for param_name, param in signature.parameters.items():
            # Ignorar parâmetros do tipo self/cls para métodos
            if (
                param_name in ("self", "cls")
                and param.kind == inspect.Parameter.POSITIONAL_OR_KEYWORD
            ):
                continue

            param_info: dict[str, object] = {"type": "object"}

            # Adicionar informações de tipo se disponíveis
            if param.annotation != inspect.Parameter.empty:
                param_type = (
                    str(param.annotation).replace("<class '", "").replace("'>", "")
                )
                param_info["type"] = param_type

            # Adicionar valor padrão se disponível
            if param.default != inspect.Parameter.empty:
                param_info["default"] = param.default

            # Determinar se o parâmetro é obrigatório
            if param.default == inspect.Parameter.empty and param.kind in (
                inspect.Parameter.POSITIONAL_ONLY,
                inspect.Parameter.POSITIONAL_OR_KEYWORD,
            ):
                param_info["required"] = True

            parameters[param_name] = param_info

        return cls(
            name=name,
            description=description,
            callable_ref=_callable,
            parameters=parameters,
        )
